@@ -21,7 +21,6 @@ from neutron.common import utils
 from neutron_lib import constants as n_const
 from oslo_concurrency import lockutils
 from oslo_log import log
-from oslo_utils import excutils
 from oslo_utils import uuidutils
 from ovsdbapp.backend.ovs_idl import event as row_event
 from ovsdbapp.backend.ovs_idl import vlog
@@ -36,6 +35,7 @@ from networking_ovn.common import constants as ovn_const
 
 LOG = log.getLogger(__name__)
 _SYNC_STATE_LOCK = lockutils.ReaderWriterLock()
+CHASSIS_METADATA_LOCK = 'chassis_metadata_lock'
 
 
 NS_PREFIX = 'ovnmeta-'
@@ -45,6 +45,7 @@ METADATA_DEFAULT_CIDR = '%s/%d' % (METADATA_DEFAULT_IP,
                                    METADATA_DEFAULT_PREFIX)
 METADATA_PORT = 80
 MAC_PATTERN = re.compile(r'([0-9A-F]{2}[:-]){5}([0-9A-F]{2})', re.I)
+OVN_VIF_PORT_TYPES = ("", "external", )
 
 MetadataPortInfo = collections.namedtuple('MetadataPortInfo', ['mac',
                                                                'ip_addresses'])
@@ -59,41 +60,80 @@ def _sync_lock(f):
     return wrapped
 
 
-def _wait_if_syncing(f):
-    """Decorator to wait if any sync operations are in progress."""
-    @six.wraps(f)
-    def wrapped(*args, **kwargs):
-        with _SYNC_STATE_LOCK.read_lock():
-            return f(*args, **kwargs)
-    return wrapped
+class ConfigException(Exception):
+    """Misconfiguration of the agent
+
+    This exception is raised when agent detects its wrong configuration.
+    Typically agent should resync when this is raised.
+    """
 
 
 class PortBindingChassisEvent(row_event.RowEvent):
-    def __init__(self, metadata_agent):
+    def __init__(self, metadata_agent, events):
         self.agent = metadata_agent
         table = 'Port_Binding'
-        events = (self.ROW_UPDATE,)
         super(PortBindingChassisEvent, self).__init__(
             events, table, None)
         self.event_name = 'PortBindingChassisEvent'
 
-    @_wait_if_syncing
     def run(self, event, row, old):
         # Check if the port has been bound/unbound to our chassis and update
         # the metadata namespace accordingly.
-        # Type must be empty to make sure it's a VIF.
-        if row.type != "":
+        resync = False
+        if row.type not in OVN_VIF_PORT_TYPES:
             return
-        new_chassis = getattr(row, 'chassis', [])
-        old_chassis = getattr(old, 'chassis', [])
-        if new_chassis and new_chassis[0].name == self.agent.chassis:
-            LOG.info("Port %s in datapath %s bound to our chassis",
-                     row.logical_port, str(row.datapath.uuid))
-            self.agent.update_datapath(str(row.datapath.uuid))
-        elif old_chassis and old_chassis[0].name == self.agent.chassis:
-            LOG.info("Port %s in datapath %s unbound from our chassis",
-                     row.logical_port, str(row.datapath.uuid))
-            self.agent.update_datapath(str(row.datapath.uuid))
+        with _SYNC_STATE_LOCK.read_lock():
+            try:
+                LOG.info(self.LOG_MSG, row.logical_port,
+                         str(row.datapath.uuid))
+                self.agent.update_datapath(str(row.datapath.uuid))
+            except ConfigException:
+                # We're now in the reader lock mode, we need to exit the
+                # context and then use writer lock
+                resync = True
+        if resync:
+            self.agent.resync()
+
+
+class PortBindingChassisCreatedEvent(PortBindingChassisEvent):
+    LOG_MSG = "Port %s in datapath %s bound to our chassis"
+
+    def __init__(self, metadata_agent):
+        events = (self.ROW_UPDATE,)
+        super(PortBindingChassisCreatedEvent, self).__init__(
+            metadata_agent, events)
+
+    def match_fn(self, event, row, old):
+        try:
+            return (row.chassis[0].name == self.agent.chassis and
+                    not old.chassis)
+        except (IndexError, AttributeError):
+            return False
+
+
+class PortBindingChassisDeletedEvent(PortBindingChassisEvent):
+    LOG_MSG = "Port %s in datapath %s unbound from our chassis"
+
+    def __init__(self, metadata_agent):
+        events = (self.ROW_UPDATE, self.ROW_DELETE)
+        super(PortBindingChassisDeletedEvent, self).__init__(
+            metadata_agent, events)
+
+    def match_fn(self, event, row, old):
+        try:
+            if event == self.ROW_UPDATE:
+                return (old.chassis[0].name == self.agent.chassis and
+                        not row.chassis)
+            else:
+                if row.chassis[0].name == self.agent.chassis:
+                    if row.type != "external":
+                        LOG.warning(
+                            'Removing non-external type port %(port_id)s with '
+                            'type "%(type)s"',
+                            {"port_id": row.uuid, "type": row.type})
+                    return True
+        except (IndexError, AttributeError):
+            return False
 
 
 class ChassisCreateEvent(row_event.RowEvent):
@@ -118,6 +158,10 @@ class ChassisCreateEvent(row_event.RowEvent):
         if self.first_time:
             self.first_time = False
         else:
+            # NOTE(lucasagomes): Re-register the ovn metadata agent
+            # with the local chassis in case its entry was re-created
+            # (happens when restarting the ovn-controller)
+            self.agent.register_metadata_agent()
             LOG.info("Connection to OVSDB established, doing a full sync")
             self.agent.sync()
 
@@ -146,30 +190,47 @@ class MetadataAgent(object):
             config=self.conf,
             resource_type='metadata')
 
+    def _load_config(self):
+        self.chassis = self._get_own_chassis_name()
+        self.ovn_bridge = self._get_ovn_bridge()
+        LOG.debug("Loaded chassis %s and ovn bridge %s.",
+                  self.chassis, self.ovn_bridge)
+
+    @_sync_lock
+    def resync(self):
+        """Resync the agent.
+
+        Reload the configuration and sync the agent again.
+        """
+        self._load_config()
+        self.sync()
+
     def start(self):
+        # Open the connection to OVS database
+        self.ovs_idl = ovsdb.MetadataAgentOvsIdl().start()
+        self._load_config()
+
         # Launch the server that will act as a proxy between the VM's and Nova.
         proxy = metadata_server.UnixDomainMetadataProxy(self.conf)
         proxy.run()
 
-        # Open the connection to OVS database
-        self.ovs_idl = ovsdb.MetadataAgentOvsIdl().start()
-        self.chassis = self._get_own_chassis_name()
-        self.ovn_bridge = self._get_ovn_bridge()
-
         # Open the connection to OVN SB database.
         self.sb_idl = ovsdb.MetadataAgentOvnSbIdl(
-            [PortBindingChassisEvent(self), ChassisCreateEvent(self),
-                SbGlobalUpdateEvent(self)]).start()
+            chassis=self.chassis,
+            events=[PortBindingChassisCreatedEvent(self),
+                    PortBindingChassisDeletedEvent(self),
+                    ChassisCreateEvent(self),
+                    SbGlobalUpdateEvent(self)]).start()
 
         # Do the initial sync.
         self.sync()
 
         # Register the agent with its corresponding Chassis
-        self._register_metadata_agent()
+        self.register_metadata_agent()
 
         proxy.wait()
 
-    def _register_metadata_agent(self):
+    def register_metadata_agent(self):
         # NOTE(lucasagomes): db_add() will not overwrite the UUID if
         # it's already set.
         ext_ids = {
@@ -212,7 +273,9 @@ class MetadataAgent(object):
         which were serving metadata but are no longer needed.
         """
         metadata_namespaces = self.ensure_all_networks_provisioned()
-        system_namespaces = ip_lib.list_network_namespaces()
+        system_namespaces = tuple(
+            ns.decode('utf-8') if isinstance(ns, bytes) else ns
+            for ns in ip_lib.list_network_namespaces())
         unused_namespaces = [ns for ns in system_namespaces if
                              ns.startswith(NS_PREFIX) and
                              ns not in metadata_namespaces]
@@ -232,6 +295,9 @@ class MetadataAgent(object):
     def _get_namespace_name(datapath):
         return NS_PREFIX + datapath
 
+    def _vif_ports(self, ports):
+        return (p for p in ports if p.type in OVN_VIF_PORT_TYPES)
+
     def teardown_datapath(self, datapath):
         """Unprovision this datapath to stop serving metadata.
 
@@ -270,7 +336,7 @@ class MetadataAgent(object):
           for this datapath.
         """
         ports = self.sb_idl.get_ports_on_chassis(self.chassis)
-        datapath_ports = [p for p in ports if p.type == '' and
+        datapath_ports = [p for p in self._vif_ports(ports) if
                           str(p.datapath.uuid) == datapath]
         if datapath_ports:
             self.provision_datapath(datapath)
@@ -366,9 +432,9 @@ class MetadataAgent(object):
         try:
             ovs_bridges.remove(self.ovn_bridge)
         except KeyError:
-            with excutils.save_and_reraise_exception():
-                LOG.exception("Configured OVN bridge %s cannot be found in "
-                              "the system.", self.ovn_bridge)
+            LOG.warning("Configured OVN bridge %s cannot be found in "
+                        "the system. Resyncing the agent.", self.ovn_bridge)
+            raise ConfigException()
 
         if ovs_bridges:
             with self.ovs_idl.transaction() as txn:
@@ -402,9 +468,9 @@ class MetadataAgent(object):
         :return: A list with the namespaces that are currently serving
         metadata
         """
-        # Retrieve all ports in our Chassis with type == ''
+        # Retrieve all VIF ports in our Chassis
         ports = self.sb_idl.get_ports_on_chassis(self.chassis)
-        datapaths = {str(p.datapath.uuid) for p in ports if p.type == ''}
+        datapaths = {str(p.datapath.uuid) for p in self._vif_ports(ports)}
         namespaces = []
         # Make sure that all those datapaths are serving metadata
         for datapath in datapaths:
@@ -414,6 +480,11 @@ class MetadataAgent(object):
 
         return namespaces
 
+    # NOTE(lucasagomes): Even tho the metadata agent is a multi-process
+    # application, there's only one Southbound database IDL instance in
+    # the agent which handles the OVSDB events therefore we do not need
+    # the external=True parameter in the @synchronized decorator.
+    @lockutils.synchronized(CHASSIS_METADATA_LOCK)
     def update_chassis_metadata_networks(self, datapath, remove=False):
         """Update metadata networks hosted in this chassis.
 
