@@ -12,17 +12,16 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from neutron.conf.agent import securitygroups_rpc
+from neutron import manager
+from neutron import opts as neutron_options
+from neutron.plugins.ml2 import plugin as ml2_plugin
 from neutron_lib.agent import topics
 from neutron_lib.plugins import directory
 from oslo_config import cfg
 from oslo_db import options as db_options
 from oslo_log import log as logging
 
-from neutron.conf.agent import securitygroups_rpc
-from neutron import manager
-from neutron import opts as neutron_options
-from neutron.plugins.ml2 import plugin as ml2_plugin
-
 from networking_ovn.common import config as ovn_config
 from networking_ovn.ml2 import mech_driver
 from networking_ovn import ovn_db_sync
@@ -70,14 +69,14 @@ class OVNMechanismDriver(mech_driver.OVNMechanismDriver):
     def update_port_postcommit(self, context):
         port = context.current
         original_port = context.original
-        self.ovn_client.update_port(port, original_port)
+        self.ovn_client.update_port(port, port_object=original_port)
 
     def delete_port_precommit(self, context):
         pass
 
     def delete_port_postcommit(self, context):
         port = context.current
-        self.ovn_client.delete_port(port)
+        self.ovn_client.delete_port(port['id'], port_object=port)
 
 
 class AgentNotifierApi(object):
