@@ -73,6 +73,10 @@ ovn_opts = [
                default=180,
                help=_('Timeout in seconds for the OVSDB '
                       'connection transaction')),
+    cfg.IntOpt('ovsdb_retry_max_interval',
+               default=180,
+               help=_('Max interval in seconds between '
+                      'each retry to get the OVN NB and SB IDLs')),
     cfg.IntOpt('ovsdb_probe_interval',
                min=0,
                default=60000,
@@ -180,14 +184,39 @@ ovn_opts = [
                        "- ntp_server:,wpad:1.2.3.5 - Unset ntp_server and "
                        "set wpad\n"
                        "See the ovn-nb(5) man page for available options.")),
+    cfg.BoolOpt('ovn_emit_need_to_frag',
+                default=False,
+                help=_('Configure OVN to emit "need to frag" packets in '
+                       'case of MTU mismatch.\n'
+                       'Before enabling this configuration make sure that '
+                       'its supported by the host kernel (version >= 5.2) '
+                       'or by checking the output of the following command: \n'
+                       'ovs-appctl -t ovs-vswitchd dpif/show-dp-features '
+                       'br-int | grep "Check pkt length action".')),
+]
+
+ovs_opts = [
+    cfg.BoolOpt('igmp_snooping_enable', default=False,
+                help=_('Enable IGMP snooping for integration bridge. If this '
+                       'option is set to True, support for Internet Group '
+                       'Management Protocol (IGMP) is enabled in integration '
+                       'bridge. '
+                       'Setting this option to True will also enable Open '
+                       'vSwitch mcast-snooping-disable-flood-unregistered '
+                       'flag. This option will disable flooding of '
+                       'unregistered multicast packets to all ports. '
+                       'The switch will send unregistered multicast packets '
+                       'only to ports connected to multicast routers.')),
 ]
 
 cfg.CONF.register_opts(ovn_opts, group='ovn')
+cfg.CONF.register_opts(ovs_opts, group='OVS')
 
 
 def list_opts():
     return [
         ('ovn', ovn_opts),
+        ('OVS', ovs_opts)
     ]
 
 
@@ -227,6 +256,10 @@ def get_ovn_ovsdb_timeout():
     return cfg.CONF.ovn.ovsdb_connection_timeout
 
 
+def get_ovn_ovsdb_retry_max_interval():
+    return cfg.CONF.ovn.ovsdb_retry_max_interval
+
+
 def get_ovn_ovsdb_probe_interval():
     return cfg.CONF.ovn.ovsdb_probe_interval
 
@@ -288,3 +321,11 @@ def setup_logging():
              {'prog': sys.argv[0],
               'version': version.version_info.release_string()})
     LOG.debug("command line: %s", " ".join(sys.argv))
+
+
+def is_ovn_emit_need_to_frag_enabled():
+    return cfg.CONF.ovn.ovn_emit_need_to_frag
+
+
+def is_igmp_snooping_enabled():
+    return cfg.CONF.OVS.igmp_snooping_enable
