@@ -10,6 +10,8 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import re
+
 from neutron_lib.api.definitions import portbindings
 from neutron_lib import constants as const
 import six
@@ -163,6 +165,7 @@ FIP_ACTION_DISASSOCIATE = 'fip_disassociate'
 
 # Loadbalancer constants
 LRP_PREFIX = "lrp-"
+RE_PORT_FROM_GWC = re.compile(r'(%s)([\w-]+)_([\w-]+)' % LRP_PREFIX)
 LB_VIP_PORT_PREFIX = "ovn-lb-vip-"
 LB_EXT_IDS_LS_REFS_KEY = 'ls_refs'
 LB_EXT_IDS_LR_REF_KEY = 'lr_ref'
@@ -184,8 +187,22 @@ MAX_GW_CHASSIS = 5
 
 UNKNOWN_ADDR = 'unknown'
 
+PORT_CAP_SWITCHDEV = 'switchdev'
+
 # TODO(lucasagomes): Create constants for other LSP types
 LSP_TYPE_LOCALNET = 'localnet'
 LSP_TYPE_VIRTUAL = 'virtual'
+LSP_TYPE_EXTERNAL = 'external'
 LSP_OPTIONS_VIRTUAL_PARENTS_KEY = 'virtual-parents'
 LSP_OPTIONS_VIRTUAL_IP_KEY = 'virtual-ip'
+
+# OVN igmp options
+MCAST_SNOOP = 'mcast_snoop'
+MCAST_FLOOD_UNREGISTERED = 'mcast_flood_unregistered'
+
+HA_CHASSIS_GROUP_DEFAULT_NAME = 'default_ha_chassis_group'
+HA_CHASSIS_GROUP_HIGHEST_PRIORITY = 32767
+
+EXTERNAL_PORT_TYPES = (portbindings.VNIC_DIRECT,
+                       portbindings.VNIC_DIRECT_PHYSICAL,
+                       portbindings.VNIC_MACVTAP)
