@@ -13,6 +13,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import abc
 import inspect
 import threading
 
@@ -24,6 +25,7 @@ from neutron_lib import exceptions as n_exc
 from oslo_config import cfg
 from oslo_log import log
 from oslo_utils import timeutils
+from ovsdbapp.backend.ovs_idl import event as row_event
 
 from networking_ovn.common import config as ovn_conf
 from networking_ovn.common import constants as ovn_const
@@ -68,7 +70,60 @@ class MaintenanceThread(object):
         self._worker = self._thread = None
 
 
-class DBInconsistenciesPeriodics(object):
+def rerun_on_schema_updates(func):
+    """Tasks decorated with this will rerun upon database version updates."""
+    func._rerun_on_schema_updates = True
+    return func
+
+
+class OVNNBDBReconnectionEvent(row_event.RowEvent):
+    """Event listening to reconnections from OVN Northbound DB."""
+
+    def __init__(self, driver, version):
+        self.driver = driver
+        self.version = version
+        table = 'Connection'
+        events = (self.ROW_CREATE,)
+        super(OVNNBDBReconnectionEvent, self).__init__(events, table, None)
+        self.event_name = self.__class__.__name__
+
+    def run(self, event, row, old):
+        curr_version = self.driver.get_ovn_nbdb_version()
+        if self.version != curr_version:
+            self.driver.nbdb_schema_updated_hook()
+            self.version = curr_version
+
+
+class SchemaAwarePeriodicsBase(object):
+
+    def __init__(self, ovn_client):
+        self._nb_idl = ovn_client._nb_idl
+        self._set_schema_aware_periodics()
+        self._nb_idl.idl.notify_handler.watch_event(OVNNBDBReconnectionEvent(
+            self, self.get_ovn_nbdb_version()))
+
+    def get_ovn_nbdb_version(self):
+        return self._nb_idl.idl._db.version
+
+    def _set_schema_aware_periodics(self):
+        self._schema_aware_periodics = []
+        for name, member in inspect.getmembers(self):
+            if not inspect.ismethod(member):
+                continue
+
+            schema_upt = getattr(member, '_rerun_on_schema_updates', None)
+            if schema_upt and periodics.is_periodic(member):
+                LOG.debug('Schema aware periodic task found: '
+                          '%(owner)s.%(member)s',
+                          {'owner': self.__class__.__name__, 'member': name})
+                self._schema_aware_periodics.append(member)
+
+    @abc.abstractmethod
+    def nbdb_schema_updated_hook(self):
+        """Hook invoked upon OVN NB schema is updated."""
+
+
+class DBInconsistenciesPeriodics(SchemaAwarePeriodicsBase):
 
     def __init__(self, ovn_client):
         self._ovn_client = ovn_client
@@ -76,9 +131,11 @@ class DBInconsistenciesPeriodics(object):
         # attributes like that, perhaps we should extend the OVNClient
         # class and create an interface for the locks ?
         self._nb_idl = self._ovn_client._nb_idl
+        self._sb_idl = self._ovn_client._sb_idl
         self._idl = self._nb_idl.idl
         self._idl.set_lock('ovn_db_inconsistencies_periodics')
         self._sync_timer = timeutils.StopWatch()
+        super(DBInconsistenciesPeriodics, self).__init__(ovn_client)
 
         self._resources_func_map = {
             ovn_const.TYPE_NETWORKS: {
@@ -140,6 +197,21 @@ class DBInconsistenciesPeriodics(object):
     def has_lock(self):
         return not self._idl.is_lock_contended
 
+    def nbdb_schema_updated_hook(self):
+        if not self.has_lock:
+            return
+
+        for func in self._schema_aware_periodics:
+            LOG.debug('OVN Northbound DB schema version was updated,'
+                      'invoking "%s"', func.__name__)
+            try:
+                func()
+            except periodics.NeverAgain:
+                pass
+            except Exception:
+                LOG.exception(
+                    'Unknown error while executing "%s"', func.__name__)
+
     def _fix_create_update(self, row):
         res_map = self._resources_func_map[row.resource_type]
         admin_context = n_context.get_admin_context()
@@ -206,6 +278,7 @@ class DBInconsistenciesPeriodics(object):
     # is held by some other neutron-server instance in the cloud, we'll attempt
     # to perform the migration every 10 seconds until completed.
     @periodics.periodic(spacing=10, run_immediately=True)
+    @rerun_on_schema_updates
     def migrate_to_port_groups(self):
         """Perform the migration from Address Sets to Port Groups. """
         # TODO(dalvarez): Remove this in U cycle when we're sure that all
@@ -444,8 +517,13 @@ class DBInconsistenciesPeriodics(object):
                 elif not type_ and ovn_const.UNKNOWN_ADDR in addresses:
                     addresses.remove(ovn_const.UNKNOWN_ADDR)
 
-            self._nb_idl.lsp_set_addresses(
-                port.name, addresses=addresses).execute(check_error=True)
+            if addresses:
+                self._nb_idl.lsp_set_addresses(
+                    port.name, addresses=addresses).execute(check_error=True)
+            else:
+                self._nb_idl.db_clear(
+                    'Logical_Switch_Port', port.name,
+                    'addresses').execute(check_error=True)
 
         raise periodics.NeverAgain()
 
@@ -463,6 +541,86 @@ class DBInconsistenciesPeriodics(object):
 
         raise periodics.NeverAgain()
 
+    # A static spacing value is used here, but this method will only run
+    # once per lock due to the use of periodics.NeverAgain().
+    @periodics.periodic(spacing=600, run_immediately=True)
+    def check_for_igmp_snoop_support(self):
+        if not self.has_lock:
+            return
+
+        with self._nb_idl.transaction(check_error=True) as txn:
+            value = ('true' if ovn_conf.is_igmp_snooping_enabled()
+                     else 'false')
+            # TODO(lucasagomes): Replace this _tables usage with ls_list()
+            # if ovsdbapp >= 1.0.0 ever becomes a lower-version dependecy
+            # for stable/train
+            for ls in self._nb_idl._tables['Logical_Switch'].rows.values():
+                if ls.other_config.get(ovn_const.MCAST_SNOOP, None) == value:
+                    continue
+                txn.add(self._nb_idl.db_set(
+                    'Logical_Switch', ls.name,
+                    ('other_config', {
+                        ovn_const.MCAST_SNOOP: value,
+                        ovn_const.MCAST_FLOOD_UNREGISTERED: value})))
+
+        raise periodics.NeverAgain()
+
+    # A static spacing value is used here, but this method will only run
+    # once per lock due to the use of periodics.NeverAgain().
+    @periodics.periodic(spacing=600, run_immediately=True)
+    def check_for_ha_chassis_group_address(self):
+        # If external ports is not supported stop running
+        # this periodic task
+        if not self._ovn_client.is_external_ports_supported():
+            raise periodics.NeverAgain()
+
+        if not self.has_lock:
+            return
+
+        default_ch_grp = self._nb_idl.ha_chassis_group_add(
+            ovn_const.HA_CHASSIS_GROUP_DEFAULT_NAME, may_exist=True).execute(
+            check_error=True)
+
+        # NOTE(lucasagomes): Find the existing chassis with the highest
+        # priority and keep it as being the highest to avoid moving
+        # things around
+        try:
+            high_prio_ch = max(default_ch_grp.ha_chassis,
+                               key=lambda x: x.priority)
+        except ValueError:
+            high_prio_ch = None
+
+        all_ch = self._sb_idl.get_all_chassis()
+        gw_ch = self._sb_idl.get_gateway_chassis_from_cms_options()
+        ch_to_del = set(all_ch) - set(gw_ch)
+
+        with self._nb_idl.transaction(check_error=True) as txn:
+            for ch in ch_to_del:
+                txn.add(self._nb_idl.ha_chassis_group_del_chassis(
+                        ovn_const.HA_CHASSIS_GROUP_DEFAULT_NAME, ch,
+                        if_exists=True))
+
+            # NOTE(lucasagomes): If the high priority chassis is in
+            # the list of chassis to be added/updated. Add it first with
+            # the highest priority number possible and then add the rest
+            # (the priority of the rest of the chassis does not matter
+            # since only the highest one is active)
+            priority = ovn_const.HA_CHASSIS_GROUP_HIGHEST_PRIORITY
+            if high_prio_ch and high_prio_ch.chassis_name in gw_ch:
+                txn.add(self._nb_idl.ha_chassis_group_add_chassis(
+                        ovn_const.HA_CHASSIS_GROUP_DEFAULT_NAME,
+                        high_prio_ch.chassis_name, priority=priority))
+                gw_ch.remove(high_prio_ch.chassis_name)
+                priority -= 1
+
+            for ch in gw_ch:
+                txn.add(self._nb_idl.ha_chassis_group_add_chassis(
+                        ovn_const.HA_CHASSIS_GROUP_DEFAULT_NAME,
+                        ch, priority=priority))
+                priority -= 1
+
+        raise periodics.NeverAgain()
+
 
 class HashRingHealthCheckPeriodics(object):
 
