@@ -14,6 +14,7 @@
 
 import atexit
 import copy
+import datetime
 import functools
 import operator
 import signal
@@ -33,7 +34,6 @@ from neutron_lib import context as n_context
 from neutron_lib import exceptions as n_exc
 from neutron_lib.plugins import directory
 from neutron_lib.plugins.ml2 import api
-from neutron_lib.services.qos import constants as qos_consts
 from oslo_config import cfg
 from oslo_db import exception as os_db_exc
 from oslo_log import log
@@ -90,8 +90,6 @@ class OVNMechanismDriver(api.MechanismDriver):
     methods that are part of the database transaction.
     """
 
-    supported_qos_rule_types = [qos_consts.RULE_TYPE_BANDWIDTH_LIMIT]
-
     def initialize(self):
         """Perform driver initialization.
 
@@ -117,7 +115,7 @@ class OVNMechanismDriver(api.MechanismDriver):
             LOG.warning('Firewall driver configuration is ignored')
         self._setup_vif_port_bindings()
         self.subscribe()
-        self.qos_driver = qos_driver.OVNQosNotificationDriver.create(self)
+        self.qos_driver = qos_driver.OVNQosDriver.create(self)
         self.trunk_driver = trunk_driver.OVNTrunkDriver.create(self)
 
     @property
@@ -137,9 +135,21 @@ class OVNMechanismDriver(api.MechanismDriver):
                                                          self._sb_ovn)
         return self._ovn_client_inst
 
+    @property
+    def nb_ovn(self):
+        # NOTE (twilson): This and sb_ovn can be moved to instance variables
+        # once all references to the private versions are changed
+        return self._nb_ovn
+
+    @property
+    def sb_ovn(self):
+        return self._sb_ovn
+
     def _setup_vif_port_bindings(self):
         self.supported_vnic_types = [portbindings.VNIC_NORMAL,
-                                     portbindings.VNIC_DIRECT]
+                                     portbindings.VNIC_DIRECT,
+                                     portbindings.VNIC_DIRECT_PHYSICAL,
+                                     portbindings.VNIC_MACVTAP]
         self.vif_details = {
             portbindings.VIF_TYPE_OVS: {
                 portbindings.CAP_PORT_FILTER: self.sg_enabled
@@ -381,7 +391,8 @@ class OVNMechanismDriver(api.MechanismDriver):
         # https://bugs.launchpad.net/neutron/+bug/1739798 is fixed.
         if context._plugin_context.session.is_active:
             return
-        self._ovn_client.update_network(context.current)
+        self._ovn_client.update_network(context.current,
+                                        original_network=context.original)
 
     def delete_network_postcommit(self, context):
         """Delete a network.
@@ -663,14 +674,11 @@ class OVNMechanismDriver(api.MechanismDriver):
                       {'port_id': port['id'], 'vnic_type': vnic_type})
             return
 
-        profile = port.get(portbindings.PROFILE)
-        capabilities = []
-        if profile:
-            capabilities = profile.get('capabilities', [])
-        if (vnic_type == portbindings.VNIC_DIRECT and
-           'switchdev' not in capabilities):
+        capabilities = utils.get_port_capabilities(port)
+        if (vnic_type in ovn_const.EXTERNAL_PORT_TYPES and
+                ovn_const.PORT_CAP_SWITCHDEV not in capabilities):
             LOG.debug("Refusing to bind port due to unsupported vnic_type: %s "
-                      "with no switchdev capability", portbindings.VNIC_DIRECT)
+                      "with no switchdev capability", vnic_type)
             return
 
         # OVN chassis information is needed to ensure a valid port bind.
@@ -1057,6 +1065,14 @@ class OVNMechanismDriver(api.MechanismDriver):
 
     def ping_chassis(self):
         """Update NB_Global.nb_cfg so that Chassis.nb_cfg will increment"""
+        last_ping = self._nb_ovn.nb_global.external_ids.get(
+            ovn_const.OVN_LIVENESS_CHECK_EXT_ID_KEY)
+        if last_ping:
+            interval = max(cfg.CONF.agent_down_time // 2, 1)
+            next_ping = (timeutils.parse_isotime(last_ping) +
+                         datetime.timedelta(seconds=interval))
+            if timeutils.utcnow(with_timezone=True) < next_ping:
+                return
 
         with self._nb_ovn.create_transaction(check_error=True,
                                              bump_nb_cfg=True) as txn:
