@@ -314,6 +314,7 @@ class OvnProviderHelper(object):
 
     def _clean_lb_if_empty(self, ovn_lb, lb_id, external_ids):
         commands = []
+        lb_to_delete = False
         if OvnProviderHelper._is_lb_empty(external_ids):
             # Verify if its only OVN LB defined. If so - leave with
             # undefined protocol. If there is different for other protocol
@@ -326,10 +327,11 @@ class OvnProviderHelper(object):
                 commands.append(
                     self.ovn_nbdb_api.db_set(
                         'Load_Balancer', ovn_lb.uuid, ('protocol', [])))
-            else:
+            elif len(defined_ovn_lbs) > 1:
                 # Delete the lb.
                 commands.append(self.ovn_nbdb_api.lb_del(ovn_lb.uuid))
-        return commands
+                lb_to_delete = True
+        return (commands, lb_to_delete)
 
     def lb_delete_lrp_assoc_handler(self, row):
         try:
@@ -788,11 +790,12 @@ class OvnProviderHelper(object):
         return pool_key
 
     def _extract_member_info(self, member):
-        mem_info = ''
+        mem_info = []
         if member:
             for mem in member.split(','):
-                mem_info += str(mem.split('_')[2]) + ','
-        return mem_info[:-1]  # Remove the last ','
+                mem_ip_port = mem.split('_')[2]
+                mem_info.append(tuple(mem_ip_port.rsplit(':', 1)))
+        return mem_info
 
     def _get_member_key(self, member, old_convention=False):
         member_info = ''
@@ -863,11 +866,22 @@ class OvnProviderHelper(object):
             if pool_id not in lb_external_ids or not lb_external_ids[pool_id]:
                 continue
 
-            ips = self._extract_member_info(lb_external_ids[pool_id])
-            vip_ips[lb_vip + ':' + vip_port] = ips
+            ips = []
+            for member_ip, member_port in self._extract_member_info(
+                    lb_external_ids[pool_id]):
+                if netaddr.IPNetwork(member_ip).version == 6:
+                    ips.append('[%s]:%s' % (member_ip, member_port))
+                else:
+                    ips.append('%s:%s' % (member_ip, member_port))
+
+            if netaddr.IPNetwork(lb_vip).version == 6:
+                lb_vip = '[%s]' % lb_vip
+            vip_ips[lb_vip + ':' + vip_port] = ','.join(ips)
 
             if vip_fip:
-                vip_ips[vip_fip + ':' + vip_port] = ips
+                if netaddr.IPNetwork(vip_fip).version == 6:
+                    vip_fip = '[%s]' % vip_fip
+                vip_ips[vip_fip + ':' + vip_port] = ','.join(ips)
 
         return vip_ips
 
@@ -1220,11 +1234,14 @@ class OvnProviderHelper(object):
 
                 # Set LB protocol to undefined only if there are no more
                 # listeners and pools defined in the LB.
-                commands.extend(
-                    self._clean_lb_if_empty(
-                        ovn_lb, listener['loadbalancer_id'], external_ids))
-                commands.extend(
-                    self._refresh_lb_vips(ovn_lb.uuid, external_ids))
+                cmds, lb_to_delete = self._clean_lb_if_empty(
+                    ovn_lb, listener['loadbalancer_id'], external_ids)
+                commands.extend(cmds)
+                # Do not refresh vips if OVN LB for given protocol
+                # has pending delete operation.
+                if not lb_to_delete:
+                    commands.extend(
+                        self._refresh_lb_vips(ovn_lb.uuid, external_ids))
                 self._execute_commands(commands)
             except Exception:
                 LOG.exception(EXCEPTION_MSG, "deletion of listener")
@@ -1431,7 +1448,7 @@ class OvnProviderHelper(object):
 
             commands.extend(
                 self._clean_lb_if_empty(
-                    ovn_lb, pool['loadbalancer_id'], external_ids))
+                    ovn_lb, pool['loadbalancer_id'], external_ids)[0])
             self._execute_commands(commands)
 
             if listener_id:
