@@ -21,19 +21,6 @@ from networking_ovn.common import constants as ovn_const
 @six.add_metaclass(abc.ABCMeta)
 class API(api.API):
 
-    @abc.abstractmethod
-    def set_lswitch_ext_ids(self, name, ext_ids, if_exists=True):
-        """Create a command to set OVN lswitch external ids
-
-        :param name:      The name of the lswitch
-        :type name:       string
-        :param ext_ids    The external ids to set for the lswitch
-        :type ext_ids:    dictionary
-        :param if_exists: Do not fail if lswitch does not exist
-        :type if_exists:  bool
-        :returns:        :class:`Command` with no result
-        """
-
     @abc.abstractmethod
     def create_lswitch_port(self, lport_name, lswitch_name, may_exist=True,
                             **columns):
