@@ -71,6 +71,66 @@ class ChassisEvent(row_event.RowEvent):
         super(ChassisEvent, self).__init__(events, table, None)
         self.event_name = 'ChassisEvent'
 
+    def handle_ha_chassis_group_changes(self, event, row, old):
+        """Handle HA Chassis Group changes.
+
+        This method handles the inclusion and removal of Chassis to/from
+        the default HA Chassis Group.
+        """
+        if not self.driver._ovn_client.is_external_ports_supported():
+            return
+
+        is_gw_chassis = utils.is_gateway_chassis(row)
+        # If the Chassis being created is not a gateway, ignore it
+        if not is_gw_chassis and event == self.ROW_CREATE:
+            return
+
+        if event == self.ROW_UPDATE:
+            is_old_gw = utils.is_gateway_chassis(old)
+            if is_gw_chassis and is_old_gw:
+                return
+            elif not is_gw_chassis and is_old_gw:
+                # Chassis is not a gateway anymore, treat it as deletion
+                event = self.ROW_DELETE
+            elif is_gw_chassis and not is_old_gw:
+                # Chassis is now a gateway, treat it as creation
+                event = self.ROW_CREATE
+
+        if event == self.ROW_CREATE:
+            default_group = self.driver._nb_ovn.ha_chassis_group_get(
+                ovn_const.HA_CHASSIS_GROUP_DEFAULT_NAME).execute(
+                check_error=True)
+
+            # Find what's the lowest priority number current in the group
+            # and add the new chassis as the new lowest
+            try:
+                min_priority = min(
+                    [ch.priority for ch in default_group.ha_chassis])
+            except ValueError:
+                min_priority = ovn_const.HA_CHASSIS_GROUP_HIGHEST_PRIORITY
+
+            self.driver._nb_ovn.ha_chassis_group_add_chassis(
+                ovn_const.HA_CHASSIS_GROUP_DEFAULT_NAME, row.name,
+                priority=min_priority - 1).execute(check_error=True)
+
+        elif event == self.ROW_DELETE:
+            self.driver._nb_ovn.ha_chassis_group_del_chassis(
+                ovn_const.HA_CHASSIS_GROUP_DEFAULT_NAME,
+                row.name, if_exists=True).execute(check_error=True)
+
+    def match_fn(self, event, row, old):
+        if event != self.ROW_UPDATE:
+            return True
+        # NOTE(lucasgomes): If the external_ids column wasn't updated
+        # (meaning, Chassis "gateway" status didn't change) just returns
+        if not hasattr(old, 'external_ids'):
+            return
+        if (old.external_ids.get('ovn-bridge-mappings') !=
+                row.external_ids.get('ovn-bridge-mappings')):
+            return True
+        f = utils.is_gateway_chassis
+        return f(old) != f(row)
+
     def run(self, event, row, old):
         host = row.hostname
         phy_nets = []
@@ -81,8 +141,35 @@ class ChassisEvent(row_event.RowEvent):
             phy_nets = list(mapping_dict)
 
         self.driver.update_segment_host_mapping(host, phy_nets)
+
         if utils.is_ovn_l3(self.l3_plugin):
-            self.l3_plugin.schedule_unhosted_gateways()
+            # If chassis lost physnet or has been
+            # deleted we can limit the scope and
+            # reschedule only ports from this chassis.
+            # In other cases we need to reschedule all gw ports.
+            kwargs = {'event_from_chassis': None}
+            if event == self.ROW_DELETE:
+                kwargs['event_from_chassis'] = row.name
+            elif event == self.ROW_UPDATE:
+                old_mappings = old.external_ids.get('ovn-bridge-mappings',
+                                                    set()) or set()
+                new_mappings = row.external_ids.get('ovn-bridge-mappings',
+                                                    set()) or set()
+                if old_mappings:
+                    old_mappings = set(old_mappings.split(','))
+                if new_mappings:
+                    new_mappings = set(new_mappings.split(','))
+
+                mappings_removed = old_mappings - new_mappings
+                mappings_added = new_mappings - old_mappings
+                if mappings_removed and not mappings_added:
+                    # Mapping has been only removed. So we can
+                    # limit scope of rescheduling only to impacted
+                    # gateway chassis.
+                    kwargs['event_from_chassis'] = row.name
+            self.l3_plugin.schedule_unhosted_gateways(**kwargs)
+
+        self.handle_ha_chassis_group_changes(event, row, old)
 
 
 class PortBindingChassisUpdateEvent(row_event.RowEvent):
@@ -266,6 +353,11 @@ class OvnDbNotifyHandler(event.RowEventHandler):
 
 
 class BaseOvnIdl(connection.OvsdbIdl):
+
+    def __init__(self, remote, schema):
+        self.notify_handler = event.RowEventHandler()
+        super(BaseOvnIdl, self).__init__(remote, schema)
+
     @classmethod
     def from_server(cls, connection_string, schema_name):
         _check_and_set_ssl_files(schema_name)
@@ -273,6 +365,9 @@ class BaseOvnIdl(connection.OvsdbIdl):
         helper.register_all()
         return cls(connection_string, helper)
 
+    def notify(self, event, row, updates=None):
+        self.notify_handler.notify(event, row, updates)
+
 
 class BaseOvnSbIdl(connection.OvsdbIdl):
     @classmethod
