@@ -138,6 +138,31 @@ class TestMetadataAgent(base.TestOVNFunctionalBase):
                 external_ids={
                     ovn_const.OVN_CIDRS_EXT_ID_KEY: '192.168.122.123/24'}))
 
+    def _create_logical_switch_port(self, type_=None):
+        lswitch_name = 'ovn-' + uuidutils.generate_uuid()
+        lswitchport_name = 'ovn-port-' + uuidutils.generate_uuid()
+        # It may take some time to ovn-northd to translate from OVN NB DB to
+        # the OVN SB DB. Wait for port binding event to happen before binding
+        # the port to chassis.
+
+        pb_event = WaitForPortBindingEvent(lswitchport_name)
+        self.handler.watch_event(pb_event)
+
+        lswitch_port_columns = {}
+        if type_:
+            lswitch_port_columns['type'] = type_
+
+        with self.nb_api.transaction(check_error=True, log_errors=True) as txn:
+            txn.add(
+                self.nb_api.ls_add(lswitch_name))
+            txn.add(
+                self.nb_api.create_lswitch_port(
+                    lswitchport_name, lswitch_name, **lswitch_port_columns))
+            self._create_metadata_port(txn, lswitch_name)
+        self.assertTrue(pb_event.wait())
+
+        return lswitchport_name, lswitch_name
+
     def test_agent_resync_on_non_existing_bridge(self):
         BR_NEW = 'br-new'
         self._mock_get_ovn_br.return_value = BR_NEW
@@ -146,26 +171,10 @@ class TestMetadataAgent(base.TestOVNFunctionalBase):
         # return it, hence the agent should trigger reconfiguration and store
         # new br-new value to its attribute.
         self.assertEqual(self.OVN_BRIDGE, self.agent.ovn_bridge)
-        lswitch_name = 'ovn-' + uuidutils.generate_uuid()
-        lswitchport_name = 'ovn-port-' + uuidutils.generate_uuid()
 
-        # It may take some time to ovn-northd to translate from OVN NB DB to
-        # the OVN SB DB. Wait for port binding event to happen before binding
-        # the port to chassis.
+        lswitchport_name, _ = self._create_logical_switch_port()
 
-        pb_event = WaitForPortBindingEvent(lswitchport_name)
-        self.handler.watch_event(pb_event)
-
-        with self.nb_api.transaction(check_error=True, log_errors=True) as txn:
-            txn.add(
-                self.nb_api.ls_add(lswitch_name))
-            txn.add(
-                self.nb_api.create_lswitch_port(
-                    lswitchport_name, lswitch_name))
-            self._create_metadata_port(txn, lswitch_name)
-        self.assertTrue(pb_event.wait())
-
-        # Trigger PortBindingChassisEvent
+        # Trigger PortBindingChassisCreatedEvent
         self.sb_api.lsp_bind(lswitchport_name, self.chassis_name).execute(
             check_error=True, log_errors=True)
         exc = Exception("Agent bridge hasn't changed from %s to %s "
@@ -176,6 +185,75 @@ class TestMetadataAgent(base.TestOVNFunctionalBase):
             timeout=10,
             exception=exc)
 
+    def _test_agent_events(self, delete, type_=None):
+        m_pb_created = mock.patch.object(
+            agent.PortBindingChassisCreatedEvent, 'run').start()
+        m_pb_deleted = mock.patch.object(
+            agent.PortBindingChassisDeletedEvent, 'run').start()
+
+        lswitchport_name, lswitch_name = self._create_logical_switch_port(
+            type_)
+        self.sb_api.lsp_bind(lswitchport_name, self.chassis_name).execute(
+            check_error=True, log_errors=True)
+
+        def pb_created():
+            if m_pb_created.call_count < 1:
+                return False
+            args = m_pb_created.call_args[0]
+            self.assertEqual('update', args[0])
+            self.assertEqual(self.chassis_name, args[1].chassis[0].name)
+            self.assertFalse(args[2].chassis)
+            return True
+
+        n_utils.wait_until_true(
+            pb_created,
+            timeout=10,
+            exception=Exception(
+                "PortBindingChassisCreatedEvent didn't happen on port "
+                "binding."))
+
+        if delete:
+            self.nb_api.delete_lswitch_port(
+                lswitchport_name, lswitch_name).execute(
+                    check_error=True, log_errors=True)
+        else:
+            self.sb_api.lsp_unbind(lswitchport_name).execute(
+                check_error=True, log_errors=True)
+
+        def pb_deleted():
+            if m_pb_deleted.call_count < 1:
+                return False
+            args = m_pb_deleted.call_args[0]
+            if delete:
+                self.assertEqual('delete', args[0])
+                self.assertTrue(args[1].chassis)
+                self.assertEqual(self.chassis_name, args[1].chassis[0].name)
+            else:
+                self.assertEqual('update', args[0])
+                self.assertFalse(args[1].chassis)
+                self.assertEqual(self.chassis_name, args[2].chassis[0].name)
+            return True
+
+        n_utils.wait_until_true(
+            pb_deleted,
+            timeout=10,
+            exception=Exception(
+                "PortBindingChassisDeletedEvent didn't happen on port "
+                "unbind or delete."))
+
+        self.assertEqual(1, m_pb_deleted.call_count)
+
+    def test_agent_unbind_port(self):
+        self._test_agent_events(delete=False)
+
+    def test_agent_delete_bound_external_port(self):
+        self._test_agent_events(delete=True, type_='external')
+
+    def test_agent_delete_bound_nonexternal_port(self):
+        with mock.patch.object(agent.LOG, 'warning') as m_warn:
+            self._test_agent_events(delete=True)
+        self.assertTrue(m_warn.called)
+
     def test_agent_registration_at_chassis_create_event(self):
         chassis = self.sb_api.lookup('Chassis', self.chassis_name)
         self.assertIn(ovn_const.OVN_AGENT_METADATA_ID_KEY,
@@ -203,3 +281,33 @@ class TestMetadataAgent(base.TestOVNFunctionalBase):
             check_for_metadata,
             timeout=10,
             exception=exc)
+
+    def test_metadata_agent_only_monitors_own_chassis(self):
+        # We already have the fake chassis which we should be monitoring, so
+        # create an event looking for a change to another chassis
+        other_name = uuidutils.generate_uuid()
+        other_chassis = self.add_fake_chassis(self.FAKE_CHASSIS_HOST,
+                                              name=other_name)
+        self.assertEqual(other_chassis, other_name)
+
+        event = MetadataAgentHealthEvent(chassis=other_name, sb_cfg=-1,
+                                         timeout=0)
+        # Use the agent's sb_idl to watch for the event since it has condition
+        self.agent.sb_idl.idl.notify_handler.watch_event(event)
+        # Use the test sb_api to set other_chassis values since shouldn't exist
+        # on agent's sb_idl
+        self.sb_api.db_set(
+            'Chassis', other_chassis,
+            ('external_ids', {'test': 'value'})).execute(check_error=True)
+
+        event2 = MetadataAgentHealthEvent(chassis=self.chassis_name, sb_cfg=-1)
+        self.agent.sb_idl.idl.notify_handler.watch_event(event2)
+        # Use the test's sb_api again to send a command so we can see if it
+        # completes and short-circuit the need to wait for a timeout to pass
+        # the test. If we get the result to this, we would have gotten the
+        # previous result as well.
+        self.sb_api.db_set(
+            'Chassis', self.chassis_name,
+            ('external_ids', {'test': 'value'})).execute(check_error=True)
+        self.assertTrue(event2.wait())
+        self.assertFalse(event.wait())
