@@ -18,6 +18,7 @@ import mock
 from futurist import periodics
 from neutron.tests.unit.plugins.ml2 import test_security_group as test_sg
 from neutron_lib.db import api as db_api
+from oslo_config import cfg
 
 from networking_ovn.common import config as ovn_config
 from networking_ovn.common import constants
@@ -27,6 +28,52 @@ from networking_ovn.db import maintenance as db_maint
 from networking_ovn.db import revision as db_rev
 from networking_ovn import ovn_db_sync
 from networking_ovn.tests.unit.db import base as db_base
+from networking_ovn.tests.unit import fakes
+
+
+class TestSchemaAwarePeriodicsBase(db_base.DBTestCase):
+
+    def test__set_schema_aware_periodics(self):
+
+        class TestClass(maintenance.SchemaAwarePeriodicsBase):
+            @periodics.periodic(spacing=1)
+            @maintenance.rerun_on_schema_updates
+            def test_method_0(self):
+                pass
+
+            @periodics.periodic(spacing=1)
+            def test_method_1(self):
+                pass
+
+            @periodics.periodic(spacing=1)
+            @maintenance.rerun_on_schema_updates
+            def test_method_2(self):
+                pass
+
+        obj = TestClass(mock.Mock())
+        # Assert that test_method_0 and test_method_2 are schema
+        # aware periodics
+        self.assertEqual([obj.test_method_0, obj.test_method_2],
+                         obj._schema_aware_periodics)
+
+    @mock.patch.object(maintenance.SchemaAwarePeriodicsBase,
+                       'get_ovn_nbdb_version')
+    def test_nbdb_schema_updated_hook(self, mock_get_ver):
+        initial_ver = '1.0.0'
+        obj = mock.Mock()
+        obj.get_ovn_nbdb_version.side_effect = (initial_ver, '1.1.0')
+        obj_evt = maintenance.OVNNBDBReconnectionEvent(obj, initial_ver)
+
+        # First run() will be called with the initial version (see
+        # side_effect), so the hook should not be invoked since the
+        # versions didn't change
+        obj_evt.run('update', mock.Mock(), mock.Mock())
+        self.assertFalse(obj.nbdb_schema_updated_hook.called)
+
+        # Second run() will be called with a different version, the
+        # hook should now be invoked
+        obj_evt.run('update', mock.Mock(), mock.Mock())
+        self.assertTrue(obj.nbdb_schema_updated_hook.called)
 
 
 @mock.patch.object(maintenance.DBInconsistenciesPeriodics,
@@ -40,7 +87,7 @@ class TestDBInconsistenciesPeriodics(db_base.DBTestCase,
             self.fmt, name='net1', admin_state_up=True)['network']
         self.port = self._make_port(
             self.fmt, self.net['id'], name='port1')['port']
-        self.fake_ovn_client = mock.Mock()
+        self.fake_ovn_client = mock.MagicMock()
         self.periodic = maintenance.DBInconsistenciesPeriodics(
             self.fake_ovn_client)
         self.session = db_api.get_writer_session()
@@ -268,3 +315,96 @@ class TestDBInconsistenciesPeriodics(db_base.DBTestCase,
         incst = [mock.Mock(resource_type=constants.TYPE_NETWORKS)] * 2
         self.periodic._log_maintenance_inconsistencies(incst, [])
         self.assertFalse(mock_log.called)
+
+    def test_check_for_igmp_snoop_support(self):
+        cfg.CONF.set_override('igmp_snooping_enable', True, group='OVS')
+        nb_idl = self.fake_ovn_client._nb_idl
+        ls0 = fakes.FakeOvsdbRow.create_one_ovsdb_row(
+            attrs={'name': 'ls0',
+                   'other_config': {
+                       constants.MCAST_SNOOP: 'false',
+                       constants.MCAST_FLOOD_UNREGISTERED: 'false'}})
+        ls1 = fakes.FakeOvsdbRow.create_one_ovsdb_row(
+            attrs={'name': 'ls1',
+                   'other_config': {}})
+        ls2 = fakes.FakeOvsdbRow.create_one_ovsdb_row(
+            attrs={'name': 'ls2',
+                   'other_config': {
+                        constants.MCAST_SNOOP: 'true',
+                        constants.MCAST_FLOOD_UNREGISTERED: 'true'}})
+
+        nb_idl._tables['Logical_Switch'].rows.values.return_value = [
+            ls0, ls1, ls2]
+
+        self.assertRaises(periodics.NeverAgain,
+                          self.periodic.check_for_igmp_snoop_support)
+
+        # "ls2" is not part of the transaction because it already
+        # have the right value set
+        expected_calls = [
+            mock.call('Logical_Switch', 'ls0',
+                      ('other_config', {
+                           constants.MCAST_SNOOP: 'true',
+                           constants.MCAST_FLOOD_UNREGISTERED: 'true'})),
+            mock.call('Logical_Switch', 'ls1',
+                      ('other_config', {
+                           constants.MCAST_SNOOP: 'true',
+                           constants.MCAST_FLOOD_UNREGISTERED: 'true'})),
+        ]
+        nb_idl.db_set.assert_has_calls(expected_calls)
+
+    def test_check_for_ha_chassis_group_address_not_supported(self):
+        self.fake_ovn_client.is_external_ports_supported.return_value = False
+        self.assertRaises(periodics.NeverAgain,
+                          self.periodic.check_for_ha_chassis_group_address)
+        self.assertFalse(
+            self.fake_ovn_client._nb_idl.ha_chassis_group_add.called)
+
+    def test_check_for_ha_chassis_group_address(self):
+        self.fake_ovn_client.is_external_ports_supported.return_value = True
+        nb_idl = self.fake_ovn_client._nb_idl
+        sb_idl = self.fake_ovn_client._sb_idl
+
+        gw_chassis_0 = fakes.FakeOvsdbRow.create_one_ovsdb_row(
+            attrs={'priority': 1,
+                   'name': 'gw_chassis_0',
+                   'chassis_name': 'gw_chassis_0'})
+        gw_chassis_1 = fakes.FakeOvsdbRow.create_one_ovsdb_row(
+            attrs={'priority': 2,
+                   'name': 'gw_chassis_1',
+                   'chassis_name': 'gw_chassis_1'})
+        non_gw_chassis_0 = fakes.FakeOvsdbRow.create_one_ovsdb_row(
+            attrs={'name': 'non_gw_chassis_0'})
+        default_ha_group = fakes.FakeOvsdbRow.create_one_ovsdb_row(
+            attrs={'ha_chassis': [gw_chassis_0, gw_chassis_1]})
+
+        nb_idl.ha_chassis_group_add.return_value.execute.return_value = (
+            default_ha_group)
+        sb_idl.get_all_chassis.return_value = [
+            non_gw_chassis_0.name, gw_chassis_0.name, gw_chassis_1.name]
+        sb_idl.get_gateway_chassis_from_cms_options.return_value = [
+            gw_chassis_0.name, gw_chassis_1.name]
+
+        # Invoke the periodic method, it meant to run only once at startup
+        # so NeverAgain will be raised at the end
+        self.assertRaises(periodics.NeverAgain,
+                          self.periodic.check_for_ha_chassis_group_address)
+
+        # Make sure the non GW chassis has been removed from the
+        # default HA_CHASSIS_GROUP
+        nb_idl.ha_chassis_group_del_chassis.assert_called_once_with(
+            constants.HA_CHASSIS_GROUP_DEFAULT_NAME, non_gw_chassis_0.name,
+            if_exists=True)
+
+        # Assert the GW chassis are being added to the
+        # default HA_CHASSIS_GROUP
+        expected_calls = [
+            mock.call(constants.HA_CHASSIS_GROUP_DEFAULT_NAME,
+                      gw_chassis_1.chassis_name,
+                      priority=constants.HA_CHASSIS_GROUP_HIGHEST_PRIORITY),
+            # Note that the second chassis is getting priority -1
+            mock.call(constants.HA_CHASSIS_GROUP_DEFAULT_NAME,
+                      gw_chassis_0.chassis_name,
+                      priority=constants.HA_CHASSIS_GROUP_HIGHEST_PRIORITY - 1)
+        ]
+        nb_idl.ha_chassis_group_add_chassis.assert_has_calls(expected_calls)
