@@ -50,8 +50,8 @@ class FakeOvsdbNbOvnIdl(object):
         self._tables['NAT'] = self.nat_table
         self._tables['Port_Group'] = self.port_group_table
         self.transaction = mock.MagicMock()
+        self.create_transaction = mock.MagicMock()
         self.ls_add = mock.Mock()
-        self.set_lswitch_ext_ids = mock.Mock()
         self.ls_del = mock.Mock()
         self.create_lswitch_port = mock.Mock()
         self.set_lswitch_port = mock.Mock()
@@ -76,6 +76,7 @@ class FakeOvsdbNbOvnIdl(object):
         self.delete_address_set = mock.Mock()
         self.update_address_set = mock.Mock()
         self.get_all_chassis_gateway_bindings = mock.Mock()
+        self.get_chassis_gateways = mock.Mock()
         self.get_gateway_chassis_binding = mock.Mock()
         self.get_unhosted_gateways = mock.Mock()
         self.add_dhcp_options = mock.Mock()
@@ -145,6 +146,10 @@ class FakeOvsdbNbOvnIdl(object):
         self.set_lswitch_port_to_virtual_type = mock.Mock()
         self.unset_lswitch_port_to_virtual_type = mock.Mock()
         self.ls_get = mock.Mock()
+        self.check_liveness = mock.Mock()
+        self.ha_chassis_group_get = mock.Mock()
+        self.qos_add = mock.Mock()
+        self.qos_del = mock.Mock()
 
 
 class FakeOvsdbSbOvnIdl(object):
