@@ -1193,8 +1193,7 @@ class OVNL3RouterPlugin(test_mech_driver.OVNMechanismDriverTestCase):
     @mock.patch('neutron.db.l3_db.L3_NAT_dbonly_mixin._get_floatingip')
     @mock.patch('neutron.db.extraroute_db.ExtraRoute_dbonly_mixin.'
                 'update_floatingip')
-    def _test_update_floatingip_associate_distributed(self, network_type,
-                                                      uf, gf, gp, gn):
+    def test_update_floatingip_associate_distributed(self, uf, gf, gp, gn):
         self.l3_inst._ovn.is_col_present.return_value = True
         self.fake_floating_ip.update({'fixed_port_id': None})
         gp.return_value = {'mac_address': '00:01:02:03:04:05',
@@ -1203,7 +1202,7 @@ class OVNL3RouterPlugin(test_mech_driver.OVNMechanismDriverTestCase):
         uf.return_value = self.fake_floating_ip_new
 
         fake_network_vlan = self.fake_network
-        fake_network_vlan[pnet.NETWORK_TYPE] = network_type
+        fake_network_vlan[pnet.NETWORK_TYPE] = constants.TYPE_FLAT
         gn.return_value = fake_network_vlan
 
         config.cfg.CONF.set_override(
@@ -1218,23 +1217,11 @@ class OVNL3RouterPlugin(test_mech_driver.OVNMechanismDriverTestCase):
             ovn_const.OVN_ROUTER_NAME_EXT_ID_KEY: utils.ovn_name(
                 self.fake_floating_ip_new['router_id']),
             ovn_const.OVN_FIP_EXT_MAC_KEY: '00:01:02:03:04:05'}
-        if network_type == constants.TYPE_VLAN:
-            self.l3_inst._ovn.add_nat_rule_in_lrouter.assert_called_once_with(
-                'neutron-new-router-id', type='dnat_and_snat',
-                logical_ip='10.10.10.10', external_ip='192.168.0.10',
-                logical_port='new-port_id', external_ids=expected_ext_ids)
-        else:
-            self.l3_inst._ovn.add_nat_rule_in_lrouter.assert_called_once_with(
-                'neutron-new-router-id', type='dnat_and_snat',
-                logical_ip='10.10.10.10', external_ip='192.168.0.10',
-                external_mac='00:01:02:03:04:05', logical_port='new-port_id',
-                external_ids=expected_ext_ids)
-
-    def test_update_floatingip_associate_distributed_flat(self):
-        self._test_update_floatingip_associate_distributed(constants.TYPE_FLAT)
-
-    def test_update_floatingip_associate_distributed_vlan(self):
-        self._test_update_floatingip_associate_distributed(constants.TYPE_VLAN)
+        self.l3_inst._ovn.add_nat_rule_in_lrouter.assert_called_once_with(
+            'neutron-new-router-id', type='dnat_and_snat',
+            logical_ip='10.10.10.10', external_ip='192.168.0.10',
+            external_mac='00:01:02:03:04:05', logical_port='new-port_id',
+            external_ids=expected_ext_ids)
 
     @mock.patch('neutron.db.l3_db.L3_NAT_dbonly_mixin._get_floatingip')
     @mock.patch('neutron.db.extraroute_db.ExtraRoute_dbonly_mixin.'
@@ -1384,8 +1371,12 @@ class OVNL3RouterPlugin(test_mech_driver.OVNMechanismDriverTestCase):
         self.l3_inst.schedule_unhosted_gateways()
         self.nb_idl().update_lrouter_port.assert_not_called()
 
-    def test_schedule_unhosted_gateways(self):
+    @mock.patch('networking_ovn.l3.l3_ovn.OVNL3RouterPlugin.'
+                '_get_gateway_port_physnet_mapping')
+    def test_schedule_unhosted_gateways(self, get_gppm):
         unhosted_gws = ['lrp-foo-1', 'lrp-foo-2', 'lrp-foo-3']
+        get_gppm.return_value = {k[len(ovn_const.LRP_PREFIX):]: 'physnet1'
+                                 for k in unhosted_gws}
         chassis_mappings = {
             'chassis1': ['physnet1'],
             'chassis2': ['physnet1'],
@@ -1435,6 +1426,34 @@ class OVNL3RouterPlugin(test_mech_driver.OVNMechanismDriverTestCase):
             mock.call('lrp-foo-3',
                       gateway_chassis=['chassis3', 'chassis2', 'chassis1'])])
 
+    @mock.patch('networking_ovn.l3.l3_ovn.OVNL3RouterPlugin.'
+                '_get_gateway_port_physnet_mapping')
+    def test_schedule_unhosted_gateways_on_event_no_gw_chassis(self, get_gppm):
+        unhosted_gws = ['lrp-foo-1', 'lrp-foo-2', 'lrp-foo-3']
+        get_gppm.return_value = {k[len(ovn_const.LRP_PREFIX):]: 'physnet1'
+                                 for k in unhosted_gws}
+        self.nb_idl().get_chassis_gateways.return_value = []
+        self.l3_inst.schedule_unhosted_gateways(event_from_chassis='chassis4')
+        self.nb_idl().get_unhosted_gateways.assert_not_called()
+
+    @mock.patch('networking_ovn.l3.l3_ovn.OVNL3RouterPlugin.'
+                '_get_gateway_port_physnet_mapping')
+    def test_schedule_unhosted_gateways_on_event(self, get_gppm):
+        unhosted_gws = ['lrp-foo-1', 'lrp-foo-2', 'lrp-foo-3']
+        get_gppm.return_value = {k[len(ovn_const.LRP_PREFIX):]: 'physnet1'
+                                 for k in unhosted_gws}
+        foo_gw = fakes.FakeOvsdbRow.create_one_ovsdb_row(
+            attrs={'name': 'lrp-foo-1_chassis1',
+                   'chassis_name': 'chassis1'})
+        self.nb_idl().get_chassis_gateways.return_value = [
+            foo_gw]
+        self.nb_idl().get_unhosted_gateways.return_value = []
+        # Fake that rescheduling is executed on chassis event
+        self.l3_inst.schedule_unhosted_gateways(event_from_chassis='chassis1')
+        # Validate that only foo-1 port is beign rescheduled.
+        self.nb_idl().get_unhosted_gateways.assert_called_once_with(
+            {'foo-1': 'physnet1'}, mock.ANY, mock.ANY)
+
     @mock.patch('neutron.db.db_base_plugin_v2.NeutronDbPluginV2.get_network')
     @mock.patch('neutron.db.db_base_plugin_v2.NeutronDbPluginV2.get_port')
     @mock.patch('neutron.db.db_base_plugin_v2.NeutronDbPluginV2.get_subnet')
