@@ -30,7 +30,7 @@ from ovsdbapp.backend.ovs_idl import idlutils
 from networking_ovn.common import config as ovn_config
 from networking_ovn.common import constants as ovn_const
 from networking_ovn.common import hash_ring_manager
-from networking_ovn.common import utils
+from networking_ovn.common import utils as ovn_utils
 from networking_ovn.db import hash_ring as db_hash_ring
 from networking_ovn.ovsdb import ovsdb_monitor
 from networking_ovn.tests import base
@@ -253,17 +253,15 @@ class TestOvnSbIdlNotifyHandler(test_mech_driver.OVNMechanismDriverTestCase):
         self._test_chassis_helper('create', self.row_json)
         self.driver.update_segment_host_mapping.assert_called_once_with(
             'fake-hostname', ['fake-phynet1'])
-        self.assertEqual(
-            1,
-            self.l3_plugin.schedule_unhosted_gateways.call_count)
+        self.l3_plugin.schedule_unhosted_gateways.assert_called_once_with(
+            event_from_chassis=None)
 
     def test_chassis_delete_event(self):
         self._test_chassis_helper('delete', self.row_json)
         self.driver.update_segment_host_mapping.assert_called_once_with(
             'fake-hostname', [])
-        self.assertEqual(
-            1,
-            self.l3_plugin.schedule_unhosted_gateways.call_count)
+        self.l3_plugin.schedule_unhosted_gateways.assert_called_once_with(
+            event_from_chassis='fake-name')
 
     def test_chassis_update_event(self):
         old_row_json = copy.deepcopy(self.row_json)
@@ -272,9 +270,58 @@ class TestOvnSbIdlNotifyHandler(test_mech_driver.OVNMechanismDriverTestCase):
         self._test_chassis_helper('update', self.row_json, old_row_json)
         self.driver.update_segment_host_mapping.assert_called_once_with(
             'fake-hostname', ['fake-phynet1'])
-        self.assertEqual(
-            1,
-            self.l3_plugin.schedule_unhosted_gateways.call_count)
+        self.l3_plugin.schedule_unhosted_gateways.assert_called_once_with(
+            event_from_chassis=None)
+
+    def test_chassis_update_event_reschedule_not_needed(self):
+        self.row_json['external_ids'][1].append(['foo_field', 'foo_value_new'])
+        old_row_json = copy.deepcopy(self.row_json)
+        old_row_json['external_ids'][1][1][1] = (
+            "foo_value")
+        self._test_chassis_helper('update', self.row_json, old_row_json)
+        self.driver.update_segment_host_mapping.assert_not_called()
+        self.l3_plugin.schedule_unhosted_gateways.assert_not_called()
+
+    def test_chassis_update_event_reschedule_lost_physnet(self):
+        old_row_json = copy.deepcopy(self.row_json)
+        self.row_json['external_ids'][1][0][1] = ''
+        self._test_chassis_helper('update', self.row_json, old_row_json)
+        self.l3_plugin.schedule_unhosted_gateways.assert_called_once_with(
+            event_from_chassis='fake-name')
+
+    def test_chassis_update_event_reschedule_add_physnet(self):
+        old_row_json = copy.deepcopy(self.row_json)
+        self.row_json['external_ids'][1][0][1] += ',foo_physnet:foo_br'
+        self._test_chassis_helper('update', self.row_json, old_row_json)
+        self.assertEqual(self.driver.update_segment_host_mapping.call_count, 1)
+        calls = self.driver.update_segment_host_mapping.mock_calls
+        self.assertEqual(calls[0][1][0], 'fake-hostname')
+        self.assertListEqual(
+            sorted(calls[0][1][1]),
+            sorted(['fake-phynet1', 'foo_physnet']))
+        self.l3_plugin.schedule_unhosted_gateways.assert_called_once_with(
+            event_from_chassis=None)
+
+    def test_chassis_update_event_reschedule_add_and_remove_physnet(self):
+        old_row_json = copy.deepcopy(self.row_json)
+        self.row_json['external_ids'][1][0][1] = 'foo_physnet:foo_br'
+        self._test_chassis_helper('update', self.row_json, old_row_json)
+        self.driver.update_segment_host_mapping.assert_called_once_with(
+            'fake-hostname', ['foo_physnet'])
+        self.l3_plugin.schedule_unhosted_gateways.assert_called_once_with(
+            event_from_chassis=None)
+
+    def test_chassis_update_empty_no_external_ids(self):
+        old_row_json = copy.deepcopy(self.row_json)
+        old_row_json.pop('external_ids')
+        with mock.patch(
+            'networking_ovn.ovsdb.'
+            'ovsdb_monitor.ChassisEvent.'
+                'handle_ha_chassis_group_changes') as mock_ha:
+            self._test_chassis_helper('update', self.row_json, old_row_json)
+            self.driver.update_segment_host_mapping.assert_not_called()
+            self.l3_plugin.schedule_unhosted_gateways.assert_not_called()
+            mock_ha.assert_not_called()
 
 
 class TestOvnDbNotifyHandler(base.TestCase):
@@ -375,61 +422,6 @@ class TestOvnConnection(base.TestCase):
                                     schema='OVN_Southbound')
 
 
-class TestChassisMetadataAgentEvent(base.TestCase):
-
-    def setUp(self):
-        super(TestChassisMetadataAgentEvent, self).setUp()
-        self.event = ovsdb_monitor.ChassisMetadataAgentEvent()
-
-        patcher = mock.patch.object(ovsdb_monitor.stats, 'AgentStats')
-        self.agent_stats = patcher.start()
-        self.addCleanup(patcher.stop)
-
-    def test_run(self):
-        row = fakes.FakeOvsdbRow.create_one_ovsdb_row(
-            attrs={'external_ids': {
-                ovn_const.OVN_AGENT_METADATA_SB_CFG_KEY: '1'}})
-
-        self.event.run(mock.ANY, row, mock.ANY)
-        self.agent_stats.add_stat.assert_called_once_with(
-            utils.ovn_metadata_name(row.uuid), 1)
-
-    def test_run_no_metadata_cfg(self):
-        row = fakes.FakeOvsdbRow.create_one_ovsdb_row()
-        self.assertIsNone(self.event.run(mock.ANY, row, mock.ANY))
-
-    def test_match_fn_event_create(self):
-        self.assertTrue(self.event.match_fn(ROW_CREATE, mock.ANY))
-
-    def test_match_fn_missing_metadata_key(self):
-        row = fakes.FakeOvsdbRow.create_one_ovsdb_row(
-            attrs={'external_ids': {
-                ovn_const.OVN_AGENT_METADATA_SB_CFG_KEY: '1'}})
-        old = fakes.FakeOvsdbRow.create_one_ovsdb_row()
-
-        self.assertFalse(self.event.match_fn(ROW_UPDATE, row, old=old))
-
-    def test_match_fn_nb_cfg_match(self):
-        row = fakes.FakeOvsdbRow.create_one_ovsdb_row(
-            attrs={'external_ids': {
-                ovn_const.OVN_AGENT_METADATA_SB_CFG_KEY: '1'}})
-        old = fakes.FakeOvsdbRow.create_one_ovsdb_row(
-            attrs={'external_ids': {
-                ovn_const.OVN_AGENT_METADATA_SB_CFG_KEY: '1'}})
-
-        self.assertFalse(self.event.match_fn(ROW_UPDATE, row, old=old))
-
-    def test_match_fn_nb_cfg_doesnt_match(self):
-        row = fakes.FakeOvsdbRow.create_one_ovsdb_row(
-            attrs={'external_ids': {
-                ovn_const.OVN_AGENT_METADATA_SB_CFG_KEY: '2'}})
-        old = fakes.FakeOvsdbRow.create_one_ovsdb_row(
-            attrs={'external_ids': {
-                ovn_const.OVN_AGENT_METADATA_SB_CFG_KEY: '1'}})
-
-        self.assertTrue(self.event.match_fn(ROW_UPDATE, row, old=old))
-
-
 class TestOvnIdlDistributedLock(base.TestCase):
 
     def setUp(self):
@@ -535,3 +527,80 @@ class TestPortBindingChassisUpdateEvent(base.TestCase):
                              'type': '_fake_', 'logical_port': 'foo'}),
             ovsdb_row(attrs={'_table': pbtable, 'chassis': 'two',
                              'type': '_fake_'}))
+
+
+class TestChassisEvent(base.TestCase):
+
+    def setUp(self):
+        super(TestChassisEvent, self).setUp()
+        self.driver = mock.Mock()
+        self.nb_ovn = self.driver._nb_ovn
+        self.driver._ovn_client.is_external_ports_supported.return_value = True
+        self.event = ovsdb_monitor.ChassisEvent(self.driver)
+        self.is_gw_ch_mock = mock.patch.object(
+            ovn_utils, 'is_gateway_chassis').start()
+        self.is_gw_ch_mock.return_value = True
+
+    def test_handle_ha_chassis_group_changes_create_not_gw(self):
+        self.is_gw_ch_mock.return_value = False
+        # Assert chassis is ignored because it's not a gateway chassis
+        self.assertIsNone(self.event.handle_ha_chassis_group_changes(
+            self.event.ROW_CREATE, mock.Mock(), mock.Mock()))
+        self.assertFalse(self.nb_ovn.ha_chassis_group_add_chassis.called)
+        self.assertFalse(self.nb_ovn.ha_chassis_group_del_chassis.called)
+
+    def _test_handle_ha_chassis_group_changes_create(self, event):
+        row = fakes.FakeOvsdbTable.create_one_ovsdb_table(
+            attrs={'name': 'SpongeBob'})
+        ch0 = fakes.FakeOvsdbTable.create_one_ovsdb_table(
+            attrs={'priority': 10})
+        ch1 = fakes.FakeOvsdbTable.create_one_ovsdb_table(
+            attrs={'priority': 9})
+        default_grp = fakes.FakeOvsdbTable.create_one_ovsdb_table(
+            attrs={'ha_chassis': [ch0, ch1]})
+        self.nb_ovn.ha_chassis_group_get.return_value.execute.return_value = (
+            default_grp)
+        self.event.handle_ha_chassis_group_changes(event, row, mock.Mock())
+        # Assert the new chassis has been added to the default
+        # group with the lowest priority
+        self.nb_ovn.ha_chassis_group_add_chassis.assert_called_once_with(
+            ovn_const.HA_CHASSIS_GROUP_DEFAULT_NAME, 'SpongeBob', priority=8)
+
+    def test_handle_ha_chassis_group_changes_create(self):
+        self._test_handle_ha_chassis_group_changes_create(
+            self.event.ROW_CREATE)
+
+    def _test_handle_ha_chassis_group_changes_delete(self, event):
+        row = fakes.FakeOvsdbTable.create_one_ovsdb_table(
+            attrs={'name': 'SpongeBob'})
+        self.event.handle_ha_chassis_group_changes(event, row, mock.Mock())
+        # Assert chassis was removed from the default group
+        self.nb_ovn.ha_chassis_group_del_chassis.assert_called_once_with(
+            ovn_const.HA_CHASSIS_GROUP_DEFAULT_NAME, 'SpongeBob',
+            if_exists=True)
+
+    def test_handle_ha_chassis_group_changes_delete(self):
+        self._test_handle_ha_chassis_group_changes_delete(
+            self.event.ROW_DELETE)
+
+    def test_handle_ha_chassis_group_changes_update_still_gw(self):
+        # Assert nothing was done because the update didn't
+        # change the gateway chassis status
+        self.assertIsNone(self.event.handle_ha_chassis_group_changes(
+            self.event.ROW_UPDATE, mock.Mock(), mock.Mock()))
+        self.assertFalse(self.nb_ovn.ha_chassis_group_add_chassis.called)
+        self.assertFalse(self.nb_ovn.ha_chassis_group_del_chassis.called)
+
+    def test_handle_ha_chassis_group_changes_update_no_longer_gw(self):
+        self.is_gw_ch_mock.side_effect = (False, True)
+        # Assert that the chassis was removed from the default group
+        # after it's no longer being a Gateway chassis
+        self._test_handle_ha_chassis_group_changes_delete(
+            self.event.ROW_UPDATE)
+
+    def test_handle_ha_chassis_group_changes_update_new_gw(self):
+        self.is_gw_ch_mock.side_effect = (True, False)
+        # Assert that the chassis was added to the default group
+        # after it became a Gateway chassis
+        self._test_handle_ha_chassis_group_changes_create(
+            self.event.ROW_UPDATE)
